#ifndef HIGH_SCORE_H
#define HIGH_SCORE_H

#include "posix_stream.h"
#include <iosfwd>
#include <map>
#include <set>
#include <string>
#include <utility>


std::string GetUserFullName();


class HighScore {
	int score_;
	std::string name_;

	friend class HighScores;
	HighScore(int score)
		: score_(score), name_(GetUserFullName()) {}
	HighScore(int score, const std::string & name)
		: score_(score), name_(name) {}
public:
	int Value() const { return score_; }
	std::string Name() const { return name_; }
};


inline bool operator<(const HighScore & lhs, const HighScore & rhs)
	{ return lhs.Value() < rhs.Value(); }


class HighScores {
	static HighScores * singleton_;

	HighScores();

	typedef std::map<std::pair<int,int>, std::multiset<HighScore> > HSCtr;
	HSCtr high_scores_;
public:
	static HighScores & Instance() {
		if (!singleton_) singleton_ = new HighScores;
		return *singleton_;
	}
	static void DestroyInstance() { delete singleton_; singleton_ = 0; }

	void Update(int score, int window_width, int window_height);
	const std::multiset<HighScore> * Get(int window_width,
	                                     int window_height) const;
private:
	void Load(std::istream &);
	void Save(std::ostream &) const;
};


#endif
